// ============================================全局数据============================================
var url_list = APP_SERVER_URL + 'office/offices';

var URL_OFFICE_TYPE = './resource/office-type.json';
var URL_PROVINCE = '../../../global/resource/province.json';
var provinceData;

var mCurrentOfficeType = 0;
var mGeocoder;

// ============================================1.初始化状态=============================================
(function () {
    mGeocoder = new BMap.Geocoder();
    loadCityData();

    initOfficeType();

    initMap($sky.Model.getDefaultCityName() || "北京");
})();

// ============================================2.事件集================================================
function search() {
    var point = SMAP.getCenter();
    $sky.bindId("place").style.display = "none";
    var name = $sky.bindId("orgName").value;
    mGeocoder.getLocation(point, function (rs) {
        var param = {
            keywords: name,
            officeType: mCurrentOfficeType,
            cityCode: getCityCode(rs.addressComponents.province, rs.addressComponents.city),
            longitude: point.lng,
            latitude: point.lat,
            pageNum: 1,
            pageSize: 32
        };
        $sky.Model.getAjax('get', url_list, param)
            .then(function (data) {
                SMAP.removeAllMarker();
                if (data.success && data.list && data.list.length > 0) {
                    data.list.forEach(function (item) {
                        if (item.latitude && item.longitude) {
                            SMAP.addInfoWindow({
                                title: item.name,
                                content: "地址：" + item.address + (item.tel ? "<br/>电话：" + item.tel : ""),
                                lat: item.latitude,
                                lng: item.longitude,
                                tel: item.tel,
                                type: item.officetype,
                                pkid: item.pkid
                            });
                        } else {
                            console.error(item);
                        }
                    });
                }
            })
            .catch(function (error) {
                console.log('error' + error);
            });
    });
}

function onFilter() {
    $(".pop").toggle();
}

/**
 * 确定
 */
function sure() {
    $(".pop").hide();
    search();
}

function cancel() {
    $(".pop").hide();
}

/**
 * 查看机构详情
 *
 * @param id   机构标识
 * @param type 机构类型（1,律所 2,公证 3,司鉴 4,法援 5,调解 6,法服 7,仲裁）
 */
function onDetailClicked(id, type) {
    var urlDetail = "";
    switch (type) {
        case "1":
            urlDetail = "../../institution/law-office/detail/law_office_detail.html";
            break;
        case "2":
            urlDetail = "../../institution/notarial-office/detail/notary_office_detail.html";
            break;
        case "3":
            urlDetail = "../../institution/judicial-auth-office/detail/judicial_auth_office_detail.html";
            break;
        case "4":
            urlDetail = "../../institution/law-support-office/detail/law_support_office_detail.html";
            break;
        case "5":
            urlDetail = "../../institution/mediation-office/detail/mediation_office_detail.html";
            break;
        case "6":
            urlDetail = "../../institution/law-service-office/detail/law_service_office_detail.html";
            break;
        case "7":
            urlDetail = "../../institution/arbitration-office/detail/arbitration_office_detail.html";
            break;
        default:
            window.SKYUI.dialog.toast('机构类别错误，type = ' + type, 'none', 5000);
            break
    }
    window.location.href = $sky.Util.urlTimestamp(urlDetail + "?pkid=" + id + "&type=" + type);
}

// ============================================3.函数集================================================

function initOfficeType() {
    $sky.Model.getAjax('get', URL_OFFICE_TYPE)
        .then(function (result) {
            var dd = {
                list: result.data
            };
            $sky.bindId("officeTypeBox").innerHTML = template('officeType', dd);

            $('.singleSelect li').click(function () {
                $(this).addClass("active").siblings().removeClass("active");
                mCurrentOfficeType = this.id.split("_")[1];
                return false;
            });
        });
}

function initMap(defaultCity) {
    SMAP.initMap("container", defaultCity, 12);
    //缩放控件
    SMAP.addNavigationControl({
        // 靠右下角位置
        anchor: BMAP_ANCHOR_BOTTOM_RIGHT,
        offset: new BMap.Size(10, 115),
        // LARGE类型
        type: BMAP_NAVIGATION_CONTROL_LARGE,
        // 启用显示定位
        enableGeolocation: true
    });
    //定位控件
    SMAP.addLocationControl({
        offset: new BMap.Size(10, 115),
        enableAutoLocation: true,
        showAddressBar: false//不显示定位信息面板
    }, onLocateCompleted, onLocationFailed);

    //停止拖拽地图时触发事件
    SMAP.addEventListener("dragend", mapDragEndListener);
    SMAP.addEventListener("load", search);
    SMAP.addEventListener("click", function (e) {
        console.log(e);
        if (!e.overlay) {
            $sky.bindId("place").style.display = "none";
        }
    });

    SMAP.addCityListControl({
        anchor: BMAP_ANCHOR_TOP_LEFT,
        offset: new BMap.Size(10, -200),
        onChangeAfter: cityChangedListener
    });
}

/**
 * 定位成功回调
 *
 * @param location
 */
function onLocateCompleted(rs) {
    console.log(rs);
}

/**
 * 定位失败回调
 */
function onLocationFailed(e) {
    console.log(e.message)
}

/**
 * 切换城市回调
 */
function cityChangedListener(rs) {
    console.log(rs);
    search();
}

/**
 * 监听地图拖动
 */
function mapDragEndListener(rs) {
    console.log(rs);
    search();
}

/**
 * 判断是否是iphone手机
 * @returns
 */
function isIphone() {
    var iphone = navigator.userAgent.toLocaleLowerCase().match(/iPhone/i);
    return iphone && iphone.length;
}

function getCityCode(province, city) {
    for (var i = 0; i < provinceData.length; i++) {
        if (provinceData[i].Name === province) {
            if (province === city) {
                return provinceData[i].Code;
            } else {
                var count = provinceData[i].level.length;
                for (var j = 0; j < count; j++) {
                    if (provinceData[i].level[j].Name === city) {
                        return provinceData[i].level[j].Code;
                    }
                }
            }
        }
    }
    return "";
}

function loadCityData() {
    $sky.Model.getAjax('get', URL_PROVINCE)
        .then(function (data) {
            provinceData = data.data;
        })
        .catch(function (error) {
            console.error(error);
        });
}